#include <UkesfSixthFormers.h>

/*
 * Title: UKESF_level_meter
 * 
 * For use with: Grove Beginner Kit for Arduino
 * 
 * About: Program to implement a spirit level by reading accelerometer data and 
 *        converting the values to roll and pitch angles to display on an OLED screen.
 *        
 * Components used:
 *        - Grove Beginner Kit for Arduino
 *            - Accelerometer
 *            - OLED display
 *        
 * Rev. 1.0
 * Author: Kasper Buckbee
 * Created: 23/05/2021
 */

//VARIABLE DECLARATIONS
//Instances of the OLED display and accelerometer
Display my_display;
Accelerometer my_accelerometer;

//Variables to store floating point numbers (i.e. decimal numbers)
float AccX, AccY, AccZ;
float roll, pitch;
float sum_of_roll, sum_of_pitch;
float average_roll, average_pitch;

//Variables to store integer numbers
int readings_counter = 0;
int num_of_samples = 10;


//SETUP function
void setup() {
  Serial.begin(9600);       //Start serial communication at 9600 baud
  my_display.begin();       //Begin the display
  my_accelerometer.begin(); //Begin the Accelerometer
}

//LOOP function
void loop() {
  //Get values for the acceleration measured in X, Y and Z dimensions
  //Note: 1 g = 9.81 m/s2
  AccX = my_accelerometer.readX();  // Read acceleration in the x axis
  AccY = my_accelerometer.readY();  // Read acceleration in the y axis
  AccZ = my_accelerometer.readZ();  // Read acceleration in the z axis

  //Calculate Roll and Pitch from the accelerometer data. 
  //See here for rigorous derivation: https://www.nxp.com/files-static/sensors/doc/app_note/AN3461.pdf
  //See here for quick derivation: https://theccontinuum.com/2012/09/24/arduino-imu-pitch-roll-from-accelerometer/ 
  roll  = (atan2(-AccY, AccZ)*180.0)/M_PI;
  pitch = (atan2(AccX, sqrt(AccY*AccY + AccZ*AccZ))*180.0)/M_PI;
  
  //Store the sum of all readings so far
  sum_of_roll = sum_of_roll + roll;
  sum_of_pitch += pitch; //Shorthand notation for adding the value of pitch to the sum
  
  //Shorthand for sayind add 1 to the counter.
  //(could also write: readings_counter = readings_counter + 1)
  readings_counter++;
  
  //This if condition is ONLY entered IF we have gathered the specified number readings
  // (i.e. 10 readings if you set num_of_samples = 10)
  if(readings_counter == num_of_samples){
    //Calculate average acceleration for each axis
    average_roll = sum_of_roll / num_of_samples;
    average_pitch = sum_of_pitch / num_of_samples;

    //Round to nearest integer value (i.e. 1.34 deg --> 1 deg etc.)
    int average_roll_rounded = round(average_roll);
    int average_pitch_rounded = round(average_pitch);

    //Print LEVEL if we are within 2 deg in both x and y
    if((abs(average_roll_rounded) <= 2) && (abs(average_pitch_rounded) <= 2)){
      my_display.printBlackLarge(3, 3, "LEVEL");
      }
    else{
      //Clear LEVEL from screen (i.e. replace with five white spaces)
      my_display.printBlackLarge(3, 3, "     ");
      }

    //Add Roll and Pitch values at the top of the screen
    my_display.printBlack(0,0,("Roll:  " + String(average_roll_rounded)+ "  "));
    my_display.printBlack(0,1,("Pitch: " + String(average_pitch_rounded)+ "  "));
    my_display.printBlack(10,0,("deg"));
    my_display.printBlack(10,1,("deg"));

    //Finally zero incrementing and summing variables to be ready for the next
    //iteration thorugh the loop
    readings_counter = 0;
    sum_of_roll = 0;
    sum_of_pitch = 0;
    average_roll = 0;
    average_pitch = 0;
  }
  //Going through the loop too quickly causes glitches, so we wait for 10ms here
  delay(10);
}
